@@ -3,6 +3,8 @@ key = {
   "Iqp": "/exchange/partition-mean-partial",
   "ql_sta": "/xpcs/sqlist",
   "ql_dyn": "/xpcs/dqlist",
+  "dqmap": "/xpcs/dqmap",
+  "sqmap": "/xpcs/sqmap",
   "type": "/xpcs/analysis_type",
   "t0": "/measurement/instrument/detector/exposure_period",
   "t1": "/measurement/instrument/detector/exposure_time",
